"""Provides CC-CEDICT traditional and simplified character constants."""

from __future__ import unicode_literals

from .simplified import CHARACTERS as simplified
from .traditional import CHARACTERS as traditional
